import { ZodObject } from 'zod';
import {} from '../Type/Response.js';
import {} from '../Interface/ActionInterface.js';
import { EnumErrorCode } from '../Type/ErrorCode.js';
import { OptionalFieldStripperHelper } from '../Helper/OptionalFieldStripperHelper.js';
export class ZodSchemaValidatedAction {
    payloadSchema;
    objectSchema;
    action;
    /**
     * Constructor.
     *
     * @param {TypePayloadSchema} payloadSchema - The Zod schema for the input data
     * @param {TypeObjectSchema} objectSchema - The Zod schema for the output data
     * @param {ActionInterface<Infer<TypePayloadSchema>, Infer<TypeObjectSchema>>} action - The action to wrap
     */
    constructor(payloadSchema, objectSchema, action) {
        this.payloadSchema = payloadSchema;
        this.objectSchema = objectSchema;
        this.action = action;
    }
    /**
     * Executes the action logic after validating the input payload and the response.
     *
     * @param {Infer<TypePayloadSchema>} payload - The input payload for the action
     * @returns {Promise<TypeResponse<Infer<TypeObjectSchema>>>} - The result of the action wrapped in a TypeResponse
     */
    async execute(payload) {
        // Validate the input using the input schema
        const parsedPayload = this.payloadSchema.safeParse(OptionalFieldStripperHelper.stripEmptyStrings(payload, this.payloadSchema));
        if (!parsedPayload.success) {
            return {
                success: false,
                messages: parsedPayload.error.errors.map((error) => ({
                    code: EnumErrorCode.VALIDATION_ERROR,
                    text: `${error.message} (at ${error.path.join('.')})`
                }))
            };
        }
        // Execute the wrapped action
        const response = await this.action.execute(parsedPayload.data);
        const validatedResponse = this.objectSchema.safeParse(response.data);
        if (!validatedResponse.success) {
            return {
                success: false,
                messages: validatedResponse.error.errors.map((error) => ({
                    code: EnumErrorCode.VALIDATION_ERROR,
                    text: `${error.message} (at ${error.path.join('.')})`
                }))
            };
        }
        // Re-assign validated output data
        response.data = validatedResponse.data;
        return response;
    }
}
