import { ZodObject } from 'zod';
import {} from '../Interface/ServiceInterface.js';
import { OptionalFieldStripperHelper } from '../Helper/OptionalFieldStripperHelper.js';
export class ZodSchemaValidatedService {
    payloadSchema;
    responseSchema;
    service;
    /**
     * Constructor.
     *
     * @param {TypePayloadSchema} payloadSchema - The Zod schema for the input data
     * @param {TypeResponseSchema} responseSchema - The Zod schema for the output data
     * @param {ServiceInterface<Infer<TypePayloadSchema>, Infer<TypeResponseSchema>>} service - The service to wrap
     */
    constructor(payloadSchema, responseSchema, service) {
        this.payloadSchema = payloadSchema;
        this.responseSchema = responseSchema;
        this.service = service;
    }
    /**
     * Executes the service logic after validating the input payload and the response.
     *
     * @param {Infer<TypePayloadSchema>} payload - The input payload for the service
     * @returns {Promise<SafeParseReturnType<any, Infer<TypeResponseSchema>>>} - The Zod validation result for the response
     */
    async execute(payload) {
        // Validate the payload using the input schema
        const parsedPayload = this.payloadSchema.safeParse(OptionalFieldStripperHelper.stripEmptyStrings(payload, this.payloadSchema));
        if (!parsedPayload.success) {
            return parsedPayload;
        }
        // Execute the wrapped service with the validated payload
        const response = await this.service.execute(parsedPayload.data);
        // Validate the response using the response schema
        const parsedResponse = this.responseSchema.safeParse(response);
        return parsedResponse;
    }
}
